package eu.mhsl.craftattack.spawn.appliances.settings;

import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.appliances.settings.datatypes.Setting;
import eu.mhsl.craftattack.spawn.appliances.settings.settings.TechnicalTablistSetting;
import net.kyori.adventure.text.Component;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.bukkit.inventory.Inventory;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.WeakHashMap;

public class Settings extends Appliance {
    public enum Key {
        TechnicalTab,
    }

    public record OpenSettingsInventory(Inventory inventory, List<Setting<?>> settings) {}
    private final WeakHashMap<Player, OpenSettingsInventory> openSettingsInventories = new WeakHashMap<>();
    private final WeakHashMap<Player, List<Setting<?>>> settingsCache = new WeakHashMap<>();

    private List<Setting<?>> getSettings(Player player) {
        if(settingsCache.containsKey(player)) return settingsCache.get(player);

        List<Setting<?>> settings =  List.of(
            new TechnicalTablistSetting()
        );

        settings.forEach(setting -> setting.initializeFromPlayer(player));
        this.settingsCache.put(player, settings);
        return settings;
    }

    public <T> T getSetting(Player player, Key key, Class<T> clazz) {
        Setting<?> setting = getSettings(player).stream()
                .filter(s -> s.getKey().equals(key))
                .findFirst()
                .orElseThrow();

        if(!clazz.equals(setting.dataType())) throw new IllegalStateException("Tried to retrieve Setting with Datatype " + clazz.getSimpleName() + " but expected " + setting.dataType().getSimpleName());
        if(!clazz.isInstance(setting.state())) throw new ClassCastException(clazz.getSimpleName() + " is not an instance of " + setting.dataType().getSimpleName());
        return clazz.cast(setting.state());
    }

    public void openSettings(Player player) {
        Inventory inventory = Bukkit.createInventory(null, 9, Component.text("Einstellungen"));
        List<Setting<?>> settings = getSettings(player);
        settings.forEach(setting -> inventory.addItem(setting.buildItem()));
        player.openInventory(inventory);
        this.openSettingsInventories.put(player, new OpenSettingsInventory(inventory, settings));
    }

    public void onSettingsClose(Player player) {
        if(!openSettingsInventories.containsKey(player)) return;
        openSettingsInventories.remove(player);
        player.updateInventory();
    }

    public boolean hasSettingsOpen(Player player) {
        return this.openSettingsInventories.containsKey(player);
    }

    public OpenSettingsInventory getOpenInventory(Player player) {
        if(!hasSettingsOpen(player)) throw new RuntimeException("Cannot retrieve data from closed Settings inventory!");
        return this.openSettingsInventories.get(player);
    }

    @Override
    protected @NotNull List<Listener> eventHandlers() {
        return List.of(new SettingsInventoryListener());
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(new SettingsCommand());
    }
}
